/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzProgressMngrLogic_h
#include "ZzProgressMngrLogic.h"
#endif //ZzProgressMngrLogic_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC ZzProgressMngrLogic

double ZzProgressMngrLogic::ExtentForLevelInCurrentBranch(SInt32 level)
{
	//vernl(6,"ZzProgressMngrLogic::ExtentForLevelInCurrentBranch(" << level << ")");
	//printf("ZzProgressMngrLogic::ExtentForLevelInCurrentBranch(" << level << ")");
	// here we must compute the extent of the (level)th level
	// level 0 is always 1.0
	double fraction = 1.0;
	for(SInt32 i = 0; i < level; i++)
	{
		SInt32 nbSubTasks = fNbTasksForLevelInCurrentBranch[i];
		//vernl(6,"fNbTasksForLevelInCurrentBranch[" << i << "] = " << nbSubTasks);
		//printf("fNbTasksForLevelInCurrentBranch[%d] = %d(",i,nbSubTasks);
		if(nbSubTasks == 0)
		{
			ZzAssert(i == level-1);
			//vernl(6, "returning "<< fraction);
//			printf("returning %f\n",fraction);
			return fraction;
		}
		fraction /= (double)nbSubTasks;
	}
	//vernl(6, "returning "<< fraction);
//	printf("returning %f\n",fraction);
	return fraction;
}

ZzProgressMngrLogic::ZzProgressMngrLogic(int nbSubTasks)
{
	fWholeProgress = 0.0;
	fLevel = 0;
	fCurrentAbsOffset = 0;
	fCurrentAbsScale = 1;
	dbg_fCurrentSubTaskInLevel[0] = -1; // -1 means not yet started
	fNbTasksForLevelInCurrentBranch[0] = 0;
}

void ZzProgressMngrLogic::SetNbSubTasks(SInt32 n)
{
//	printf("ZzProgressMngrLogic::SetNbSubTasks(%d)\n",n);
	// this is still in the parent level context
	//vernl(6,"BEFORE fCurrentAbsScale = " << fCurrentAbsScale);
//	printf("    BEFORE fCurrentAbsScale = %f\n",fCurrentAbsScale);
	fCurrentAbsScale = ExtentForLevelInCurrentBranch(fLevel)/(double)n;
	//vernl(6,"AFTER fCurrentAbsScale = " << fCurrentAbsScale);
//	printf("    AFTER fCurrentAbsScale = %f\n",fCurrentAbsScale);
	fNbTasksForLevelInCurrentBranch[fLevel] = n;
	dbg_fCurrentSubTaskInLevel[fLevel] = -1;
}

double ZzProgressMngrLogic::GetWholeProgress()
{
	return fWholeProgress;
}

void ZzProgressMngrLogic::SetProgress(double p)
{
	//vernl(6,"SetProgress(" << p << ")");
//	printf("ZzProgressMngrLogic::SetProgress(%f)\n",p);
	vvernl(6,fCurrentAbsOffset);
//	printf("    fCurrentAbsOffset = %f\n",fCurrentAbsOffset);
	//vvernl(6,fCurrentAbsScale);
//	printf("    fCurrentAbsScale = %f\n",fCurrentAbsScale);
	fWholeProgress = fCurrentAbsOffset + p*fCurrentAbsScale;
//	vvernl(6,fWholeProgress);
}

void ZzProgressMngrLogic::EnterSubTask()
{
//	vernl(6,"EnterSubTask()");
//	printf("EnterSubTask()\n");
	dbg_fCurrentSubTaskInLevel[fLevel]++; // parent context
	ZzAssert(dbg_fCurrentSubTaskInLevel[fLevel] < fNbTasksForLevelInCurrentBranch[fLevel]); // parent context
	fLevel++; // go into child context
	fNbTasksForLevelInCurrentBranch[fLevel] = 0; // by default, a task has 0 sub tasks
}

void ZzProgressMngrLogic::LeaveSubTask()
{
	//vernl(6,"LeaveSubTask()");
	//printf("LeaveSubTask()\n");
	if(fNbTasksForLevelInCurrentBranch[fLevel] == 0)
	{
		//vernl(6,"fNbTasksForLevelInCurrentBranch[fLevel] == 0 (this is a concrete level) thus we need to increment abspos");
		//printf("fNbTasksForLevelInCurrentBranch[fLevel] == 0 (this is a concrete level) thus we need to increment abspos");
		//vernl(6,"fLevel at beginning = " << fLevel);
		//printf("fLevel at beginning = " << fLevel);
		//printf("ZzProgressMngrLogic::LeaveSubTask() fLevel = %d\n",fLevel);
		//vernl(6,"before incr, fCurrentAbsOffset = " << fCurrentAbsOffset);
		double extentForThisLevel = ExtentForLevelInCurrentBranch(fLevel);
		//printf("ExtentForLevelInCurrentBranch(fLevel) = %f\n",extentForThisLevel);
		fCurrentAbsOffset += extentForThisLevel; // this is the extent for the child context
		//vernl(6,"after incr, fCurrentAbsOffset = " << fCurrentAbsOffset);
		if(fCurrentAbsOffset > 1.01)
		{
	//		vernl(0,"ALERT : fCurrentAbsOffset > 1.01 " << fCurrentAbsOffset);
		}
	}
	fLevel--;
}


